/*
 *    Sample database table partioning example
 *    David Klee 
 *    2013.01.11
 *
 *    Creates a sample database, new table, and partitioning configuration to properly 
 *    partition data based on the month.
 */

 --create a new sample database
CREATE DATABASE [PartitionTest1]
 CONTAINMENT = NONE
 ON  PRIMARY 
( NAME = N'PartitionTest', FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTest.mdf' , SIZE = 4096KB , FILEGROWTH = 1024KB )
 LOG ON 
( NAME = N'PartitionTest_log', FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTest_log.ldf' , SIZE = 1024KB , FILEGROWTH = 10%)
GO

--add file groups for future data placement flexibility
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth01   
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth02 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth03 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth04 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth05 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth06 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth07 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth08 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth09 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth10 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth11 
ALTER DATABASE PartitionTest1 ADD FILEGROUP FGMonth12
go

--add new database data files corresponding to the new file groups
ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month01',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth01.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth01]	
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month02',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth02.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth02]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month03',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth03.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth03]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month04',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth04.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth04]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month05',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth05.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth05]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month06',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth06.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth06]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month07',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth07.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth07]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month08',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth08.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth08]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month09',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth09.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth09]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month10',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth10.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth10]
go

ALTER DATABASE PartitionTest1 

	ADD FILE (NAME = N'Month11',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth11.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth11]
go

ALTER DATABASE PartitionTest1  
	ADD FILE (NAME = N'Month12',    
	FILENAME = N'E:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\PartitionTestMonth12.ndf',    
	SIZE = 5MB, FILEGROWTH = 5MB)  TO FILEGROUP [FGMonth12]
go

use PartitionTest1
go

--create function for getting correct partition number from month of date
CREATE FUNCTION [dbo].[fnGetPartition](@Dt Date)
RETURNS TINYINT
WITH SCHEMABINDING
AS
BEGIN
    DECLARE @RetVal TINYINT
    SET @RetVal = datepart(mm, @dt) 
    RETURN @RetVal
END
GO

--create possible values for the partitioning function
--notice that 12 will be bundled with the right side of the upcoming partitioning scheme
CREATE PARTITION FUNCTION [pfPartition](TINYINT)
AS RANGE LEFT FOR VALUES
(
    1,2,3,4,5,6,7,8,9,10,11
)
GO

--create the partition scheme that lines up with the new file groups
CREATE PARTITION SCHEME psPartitionTable
AS PARTITION pfPartition
TO (FGMonth01,FGMonth02,FGMonth03,FGMonth04,FGMonth05,FGMonth06,FGMonth07,FGMonth08,FGMonth09,FGMonth10,FGMonth11,FGMonth12)
GO

--create the new test table
CREATE TABLE [dbo].[TestTable1](
	[PartitionNum] AS dbo.fnGetPartition(GeneratedDate) PERSISTED,
	[GeneratedDate] [date] NOT NULL
) ON psPartitionTable(PartitionNum)
GO

--create the staging table to be used for the data deletes. Note: it must be the same 
--structure as the source table.
CREATE TABLE [dbo].[StagingTable1](
	[PartitionNum] AS dbo.fnGetPartition(GeneratedDate) PERSISTED,
	[GeneratedDate] [date] NOT NULL
) ON psPartitionTable(PartitionNum)
go

--insert 5000 randomly generated dates.
--thanks http://www.latifkhan.co.uk/index.php/select-random-date-time-in-sql-server/ for the cool query!
insert into dbo.TestTable1 (GeneratedDate) 
	select cast(
	cast( getdate() as int )
	-5555 * rand( cast( cast( newid() as binary(8) ) as int ) )
	as datetime )
go 5000  --do this 5000 times. 

--random month = april (month number 4)
alter table dbo.TestTable1 
switch partition 4 to dbo.[StagingTable1] partition 4
go

--not needed.
--ALTER PARTITION SCHEME psPartitionTable
--NEXT USED FGMonth04
--GO
--ALTER PARTITION FUNCTION pfPartition()
--      SPLIT RANGE (4);
--go

SELECT *
FROM sys.partitions
WHERE OBJECT_NAME(OBJECT_ID)='TestTable1';
go

SELECT p.partition_number, fg.name, p.rows
FROM sys.partitions p
    INNER JOIN sys.allocation_units au
    ON au.container_id = p.hobt_id
    INNER JOIN sys.filegroups fg
    ON fg.data_space_id = au.data_space_id
WHERE p.object_id = OBJECT_ID('dbo.TestTable1')
go

--show that everything is now in the staging table and nothing in the partition 4 in the source table
select * from dbo.stagingtable1

--and now instant truncates replace a timed and disruptive delete process
truncate table dbo.stagingtable1
